--***************************************************************
--* File         : TB_SOADC.vhd                                 *
--* Created      : time-stamp: <09/11/14 11:30:01 kawasan>      *
--* Modified     : Time-stamp: <09/11/17 03:23:52 kawasan>      *
--*								*
--* Test-Bench                                                  *
--*								*
--* Copyright 2009 Osamu Kawashima. all rights reserved		*
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use std.textio.all;
use ieee.std_logic_textio.all;

entity TB_SOADC is end;

architecture behavior of TB_SOADC is
    
	component SOADC
	generic (
		BITSIZE         : positive := 8
	);
	port (
		RESET           : in    std_logic;
		SYSCLK          : in    std_logic;
		EN              : in    std_logic;
		DATI            : in    std_logic_vector(BITSIZE-1 downto 0);

		STB             : out   std_logic;
		SCK             : out   std_logic;
		DO              : out   std_logic
	);
	end component;

	constant BITSIZE        : positive := 8;

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	constant WT1            : Time := 10 ns;
	constant WT_DELAY       : Time := 1 ns;
	constant WT_IDLE        : Time := 100 ns;
	constant WT_AD_SMP      : Time := 62500 ns;	-- A/D-Sampling-Rate(62.5us)
	constant CLK_CYCLE      : Time := 21 ns;	-- about 48MHz

	signal RESET           : std_logic := '1';
	signal SYSCLK          : std_logic := '0';
	signal EN              : std_logic := '0';
	signal DATI            : std_logic_vector(BITSIZE-1 downto 0) := (others=>'0');
	signal STB             : std_logic := 'Z';
	signal SCK             : std_logic := 'Z';
	signal DO              : std_logic := 'Z';

begin
    
	uSOADC : SOADC
	generic map (
		BITSIZE         => BITSIZE
	)
	port map (
		RESET           => RESET,
		SYSCLK          => SYSCLK,
		EN              => EN,
		DATI            => DATI,
		STB             => STB,
		SCK             => SCK,
		DO              => DO
	);

	process begin
		wait for CLK_CYCLE/2;
		SYSCLK <= not SYSCLK;
	end process;

	process
		variable ln         : line;
	begin
		-- Reset
		wait for WT_IDLE+WT_DELAY;
		RESET <= '0';
		wait for WT_IDLE;

		-- Event-01
		wait for WT_AD_SMP;
		DATI <= "10000001";

		-- Event-02
		wait for WT_AD_SMP;
		EN <= '1';
		wait for WT_DELAY;
		DATI <= "10000010";

		-- Event-03
		wait for WT_AD_SMP;
		DATI <= "10000011";

		-- Event-04
		wait for WT_AD_SMP;
		EN <= '0';
		wait for WT_DELAY;
		DATI <= "10000100";

		-- End
		write(ln, string'("--- End of Test-Bench. ---"));
		writeline(output, ln);
		wait;
	end process;

end behavior;
